@@ -1,7 +1,7 @@
 from typing import MutableMapping, Dict
 from urllib.parse import quote, unquote
 
-from .cloudstorage.cloudstorage import CloudStorage
+from .storageproviders.storageprovider import StorageProvider
 
 
 def _safe_key(key: str) -> str:
@@ -14,30 +14,32 @@ def _unsafe_key(key: str) -> str:
     return unquote(key)
 
 
-class CloudStorageMapping(MutableMapping):
+class CloudMapping(MutableMapping):
     etags: Dict[str, str]
 
     def __init__(
         self,
-        cloudstorage: CloudStorage,
-        metadata: Dict[str, str],
+        storageprovider: StorageProvider,
+        sync_initially: bool = True,
     ) -> None:
-        self._cloudstorage = cloudstorage
+        self._storageprovider = storageprovider
         self.etags = {}
-        if self._cloudstorage.create_if_not_exists(metadata=metadata):
+        if self._storageprovider.create_if_not_exists() and sync_initially:
             self.sync_with_cloud()
 
     def sync_with_cloud(self, key: str = None) -> None:
         prefix_key = _safe_key(key) if key is not None else None
-        self.etags.update({_unsafe_key(k): i for k, i in self._cloudstorage.list_keys_and_ids(prefix_key).items()})
+        self.etags.update({_unsafe_key(k): i for k, i in self._storageprovider.list_keys_and_etags(prefix_key).items()})
 
     def __getitem__(self, key: str) -> bytes:
         if key not in self.etags:
             raise KeyError(key)
-        return self._cloudstorage.download_data(key=_safe_key(key), etag=self.etags[key])
+        return self._storageprovider.download_data(key=_safe_key(key), etag=self.etags[key])
 
     def __setitem__(self, key: str, value: bytes) -> None:
-        self.etags[key] = self._cloudstorage.upload_data(
+        if not isinstance(value, bytes):
+            raise ValueError("Value must be bytes like")
+        self.etags[key] = self._storageprovider.upload_data(
             key=_safe_key(key),
             etag=self.etags.get(key, None),
             data=value,
@@ -46,7 +48,7 @@ class CloudStorageMapping(MutableMapping):
     def __delitem__(self, key: str) -> None:
         if key not in self.etags:
             raise KeyError(key)
-        self._cloudstorage.delete_data(key=_safe_key(key), etag=self.etags[key])
+        self._storageprovider.delete_data(key=_safe_key(key), etag=self.etags[key])
         del self.etags[key]
 
     def __contains__(self, key: str) -> bool:
@@ -60,14 +62,17 @@ class CloudStorageMapping(MutableMapping):
     def __len__(self) -> int:
         return len(self.etags)
 
+    def __repr__(self) -> str:
+        return f"cloudmapping<{self._storageprovider.safe_name()}>"
+
     @classmethod
-    def create_with_buffers(cls, *io_buffers, **kwargs) -> MutableMapping:
+    def with_buffers(cls, io_buffers, *args, **kwargs) -> MutableMapping:
         from zict import Func
 
         if len(io_buffers) % 2 != 0:
             raise ValueError("Must have an equal number of input buffers as output buffers")
 
-        raw_mapping = cls(**kwargs)
+        raw_mapping = cls(*args, **kwargs)
         mapping = raw_mapping
 
         for dump, load in zip(io_buffers[::2], io_buffers[1::2]):
@@ -77,19 +82,23 @@ class CloudStorageMapping(MutableMapping):
         return mapping
 
     @classmethod
-    def create_with_pickle(
-        cls,
-        **kwargs,
-    ) -> MutableMapping:
+    def with_pickle(cls, *args, **kwargs) -> MutableMapping:
         import pickle
 
-        return cls.create_with_buffers(pickle.dumps, pickle.loads, **kwargs)
+        return cls.with_buffers([pickle.dumps, pickle.loads], *args, **kwargs)
 
     @classmethod
-    def create_with_zlib_pickle(
-        cls,
-        **kwargs,
-    ) -> MutableMapping:
-        import zlib, pickle
+    def with_json(cls, *args, **kwargs) -> MutableMapping:
+        import json
 
-        return cls.create_with_buffers(zlib.compress, zlib.decompress, pickle.dumps, pickle.loads, **kwargs)
+        return cls.with_buffers([json.dumps, json.loads], *args, **kwargs)
+
+    @classmethod
+    def with_json_zlib(cls, *args, **kwargs) -> MutableMapping:
+        import json, zlib
+
+        return cls.with_buffers(
+            [json.dumps, json.loads, zlib.compress, zlib.decompress],
+            *args,
+            **kwargs,
+        )
